package de.ozgcloud.nachrichten.postfach.osiv2;

import static org.assertj.core.api.Assertions.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.condition.EnabledIfEnvironmentVariable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.DynamicPropertyRegistry;
import org.springframework.test.context.DynamicPropertySource;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.DummyStringBasedIdentifier;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachAddressTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;

@SpringBootTest(classes = TestApplication.class, webEnvironment = SpringBootTest.WebEnvironment.NONE)
@ActiveProfiles("itcase")
@EnabledIfEnvironmentVariable(named = "SH_STAGE_CLIENT_SECRET", matches = ".+")
public class OsiPostfachRemoteServiceRemoteITCase {

	@Autowired
	private OsiPostfachRemoteService osiPostfachRemoteService;

	private final PostfachNachricht nachricht = PostfachNachrichtTestFactory.createBuilder()
			.postfachAddress(PostfachAddressTestFactory.createBuilder()
					.identifier(DummyStringBasedIdentifier.builder()
							.mailboxId("sh/sh/60e319ae-4fe7-4f47-c7c8-08dca563280a")
							.build())
					.build())
			.build();

	@DynamicPropertySource
	static void dynamicProperties(DynamicPropertyRegistry registry) {
		registry.add(
				"spring.security.oauth2.client.registration.osi2.client-secret",
				() -> System.getenv("SH_STAGE_CLIENT_SECRET")
		);
		registry.add(
				"ozgcloud.osiv2-postfach.http-proxy.host",
				() -> matchProxyRegex(System.getenv("HTTP_PROXY")).group(1)
		);
		registry.add(
				"ozgcloud.osiv2-postfach.http-proxy.port",
				() -> matchProxyRegex(System.getenv("HTTP_PROXY")).group(2)
		);
	}

	private static Matcher matchProxyRegex(String text) {
		var matcher = Pattern.compile("([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+):([0-9]+)").matcher(text);
		if (matcher.find()) {
			return matcher;
		}
		throw new IllegalArgumentException("Proxy host and port not found in '%s'".formatted(text));
	}

	@DisplayName("send message")
	@Nested
	class TestSendMessage {

		@DisplayName("should not fail")
		@Test
		void shouldNotFail() {
			assertThatCode(() -> osiPostfachRemoteService.sendMessage(nachricht))
					.doesNotThrowAnyException();
		}

	}
}
